package main;
import java.io.File;
import java.io.IOException;
import net.sf.jni4net.Bridge;
import kratos_3.network.*;
import kratos_3.runtimeplatform.*;
import kratos_3.*;
import java.util.Scanner;

public class Main {
	
	static FactSetOnDemand fsod;
	
	public static void printOutFactletResults(DataFrame df) {
        if (df.hasError())
        {
            df.throwErrorIfAnyIsPresent();
        }
        System.out.println(fsod.GetEventLogger().getOutputBuffer()); //Print a summary string
        system.Enum[] types = df.getColumnTypes();
        for (int row = 0; row < df.getNumberOfRows(); row++)
        {
            for (int col = 0; col < df.getNumberOfColumns(); col++)
            {
                Object o = df.getCellAt(row, col); // Note that we know what the type of this cell is, so we could cast it or treat it as its proper type if we needed to
                System.out.println("Row: " + row + ", Col: " + col + ", Value: " + o.toString());
            }
        }
	}
	
	// While jni4net automatically transforms base strings from java to .net for us, we need to manually convert string arrays
	public static system.String[] convertJavaStringArrayToNetArray(String[] strings) {
		system.String[] netStrings = new system.String[strings.length];
		for(int i = 0; i < strings.length; i++) {
			netStrings[i] = new system.String(strings[i]);
		}
		return netStrings;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			Bridge.setVerbose(true);
			Bridge.init();
			Bridge.LoadAndRegisterAssemblyFrom(new File("Kratos_3.j4n.dll"));
		} catch(IOException e) {
			e.printStackTrace();
		}
		
		fsod = new FactSetOnDemand();
		// Set our configuration. We definitely need to set our username and password, as given to us by FactSet Support staff
        IConfigManager config = fsod.getConfig();    /// 
        ConfigOptions ddusername = (ConfigOptions)system.Enum.Parse(ConfigOptions.typeof(), "DataDirectUserName");
        ConfigOptions ddpassword = (ConfigOptions)system.Enum.Parse(ConfigOptions.typeof(), "DataDirectPassword");
        config.setConfig(ddusername, <INSERT USERNAME HERE>);
        config.setConfig(ddpassword, <INSERT PASSWORD HERE>);
        
        
        
        DataFrame df;
        UploadFactlet uFactlet = new UploadFactlet("TestOFDB_CSharp", new String[]{"ID","Date","my_price","my_volume"}, new String[]{"Id","Date","Double","Double"});
        String[] dates = { "04/10/2013", "04/11/2013", "04/12/2013" };
        double[] fds_price = { 93.33, Double.NaN, 92.65 };
        double[] fds_volume = { 388.205, Double.NaN, 359.396 };
        double[] xom_price = { 88.68, Double.NaN, 89.22 };
        double[] xom_volume = { 14826.0, Double.NaN, 14980.0 };
        for (int i = 0; i < dates.length; i++)
        {
            uFactlet.addDataPoint("FDS");
            uFactlet.addDataPoint(dates[i]);
            uFactlet.addDataPoint(fds_price[i]);
            uFactlet.addDataPoint(fds_volume[i]);
        }
        for (int i = 0; i < dates.length; i++)
        {
            uFactlet.addDataPoint("XOM");
            uFactlet.addDataPoint(dates[i]);
            uFactlet.addDataPoint(xom_price[i]);
            uFactlet.addDataPoint(xom_volume[i]);
        }
        df = fsod.executeFactlet(uFactlet);
        df.throwErrorIfAnyIsPresent();

        // In this example, we will run ExtractFormulaHistory and examine the unique
        // IDs and Dates in the returned data
        df = fsod.ExtractFormulaHistory("xom,stl-no,fds,ibm,efc,appl,goog", "fg_eps(0,-1m,d)", "");
        printOutFactletResults(df);

        // Extract Benchmark Detail
        df = fsod.ExtractBenchmarkDetail("SP50", "items", "FG_COMPANY_NAME,FG_MKT_VALUE");
        printOutFactletResults(df);

        df = fsod.ExtractBenchmarkDetail("SP50", "items", "p_price,proper_name", convertJavaStringArrayToNetArray(new String[] { "cutoff", "10" }));
        printOutFactletResults(df);

        
        df = fsod.ExtractBenchmarkDetail("SP50", "items", "p_price", convertJavaStringArrayToNetArray(new String[] { "dates", "20110115:20110120:d" }));
        printOutFactletResults(df);
        
        System.out.println("Press enter to continue...");
        Scanner keyboard = new Scanner(System.in);
        keyboard.nextLine();
	}

}
